#
# Autoconf macro for expansion of pkgc-config glob patterns specifications
#
#                                          Anthony Mallet on Fri Mar 14 2025
#

# --- AG_EXPAND_PKG_GLOB (variable, replacements ...) ----------------------
#
AC_DEFUN([AG_EXPAND_PKG_GLOB],
[
  AC_REQUIRE([PKG_PROG_PKG_CONFIG])
  dnl
  dnl build a PKG_CHECK_MODULES specification by expanding patterns
  ag_pkg_check_modules_glob() {
    eval "$ag_unsetf"
    IFS=$ag_save_IFS
    [$1]=
    while test [$]# -gt 0; do
      dnl shift a complete ag_pkg [>= ag_req] spec
      ag_pkg="[$]1"; shift
      ag_req=""
      case "[$]1" in [[\>\<=\!]]*) ag_req=" [$]1 [$]2"; shift 2;; esac

      dnl check forced replacement
      ag_pkg_repl=
      case "$ag_pkg" in m4_foreach(repl, [m4_shift($@)], [
        m4_car(repl)@:}@ ag_pkg_repl="m4_do(m4_cdr(repl))";;
      ])
      esac
      if test -n "$ag_pkg_repl"; then
        AC_MSG_CHECKING([expansion of $ag_pkg])
        ag_pkg="$ag_pkg_repl"
        AC_MSG_RESULT([(forced) $ag_pkg])
      fi

      dnl if pkg is a glob pattern
      case "$ag_pkg" in *[[@<:@?\|*]]*)
        AC_MSG_CHECKING([expansion of $ag_pkg$ag_req])
        dnl cache available package names
        if ! test ${ag_pkg_all:+set}; then
          if test -n "$PKG_CONFIG"; then
            ag_pkg_all=
            while read ag_p ag_d; do
              ag_pkg_all="$ag_pkg_all $ag_p"
            done <<AG_EOF
`$PKG_CONFIG --list-all`
AG_EOF
          fi
        fi
        dnl check match
        for ag_p in $ag_pkg_all; do
          ag_match=
          dnl use 'eval case' so that `|' alternatives can be used in $ag_pkg
          eval >/dev/null 2>&1 \
            'case "$ag_p" in' "$ag_pkg)" 'ag_match="$ag_p";; esac'
          dnl make sure base name satisfies any ag_req
          if test -n "$ag_match"; then
            ag_pkg="$ag_p"; dnl prepare ag_pkg so that in case of error and no
                            dnl further match, this package (and error) will be
                            dnl reported below
            PKG_CHECK_EXISTS([$ag_p$ag_req], [break])
          fi
        done
        AC_MSG_RESULT([$ag_pkg$ag_req]);;
      esac
      [$1]="$[$1]${[$1]:+, }$ag_pkg$ag_req"
    done
  }
  dnl
  dnl disable globbing and add comma to IFS to process the spec argument
  ag_unsetf="$(set +o)"
  set -f
  ag_save_IFS=$IFS
  IFS=' ''	,
'
  ag_pkg_check_modules_glob $[$1]
])
